'use strict';

var generateScriptableBanner = require('@scriptables/manifest/generateScriptableBanner');
var findicloud = require('findicloud');
var promises = require('fs/promises');
var path = require('path');
var micromatch = require('micromatch');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var micromatch__default = /*#__PURE__*/_interopDefault(micromatch);

// src/scriptable-deploy.ts

// src/logger.ts
var levels = {
  verbose: 0,
  info: 1,
  warn: 2,
  error: 3
};
var Logger = class {
  namespace;
  level;
  enabled;
  constructor(namespace, options = {}) {
    this.namespace = namespace;
    this.level = options.level || "info";
    this.enabled = options.enabled ?? true;
  }
  shouldLog(msgLevel) {
    return this.enabled && levels[msgLevel] >= levels[this.level];
  }
  verbose(...args) {
    if (this.shouldLog("verbose")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  info(...args) {
    if (this.shouldLog("info")) {
      console.info(`[${this.namespace}]`, ...args);
    }
  }
  warn(...args) {
    if (this.shouldLog("warn")) {
      console.warn(`[${this.namespace}]`, ...args);
    }
  }
  error(...args) {
    if (this.shouldLog("error")) {
      console.error(`[${this.namespace}]`, ...args);
    }
  }
};
function createLogger(namespace, options) {
  return new Logger(namespace, options);
}
function createPatternMatcher(patterns) {
  if (!patterns) return () => true;
  const patternArray = Array.isArray(patterns) ? patterns : [patterns];
  const includePatterns = patternArray.filter((p) => !p.startsWith("!"));
  const excludePatterns = patternArray.filter((p) => p.startsWith("!")).map((p) => p.slice(1));
  return (fileName) => {
    return micromatch__default.default.isMatch(fileName, includePatterns, {
      dot: true,
      ignore: excludePatterns
    });
  };
}

// src/scriptable-deploy.ts
var findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
async function tryReadManifest(filePath, extensions) {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
      const content = await promises.readFile(manifestPath, "utf-8");
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}
function scriptableDeploy(options = {}) {
  const {
    verbose = false,
    continueOnError = true,
    scriptableDir,
    patterns = ["**/*.{ts,tsx,js,jsx}"],
    addBanner = true,
    manifestExtensions = [".manifest.json", ".manifest", ".json"]
  } = options;
  const filterFile = createPatternMatcher(patterns);
  const logger = createLogger("scriptable-deploy", {
    level: verbose ? "verbose" : "info"
  });
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      if (!build.initialOptions.metafile) {
        logger.verbose(verbose, "Automatically enabling metafile option");
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const [found] = await findicloud.findICloudDrivePaths({ appNamePattern: "Scriptable", types: [findicloud.PathType.APP_STORAGE] });
            targetDir = found?.path;
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          if (verbose) {
            console.log(`[scriptable-deploy] Target directory: ${targetDir}`);
          }
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger.error(message);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          logger.error("Missing required configuration");
          return;
        }
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => {
            const { entryPoint } = result.metafile.outputs[file];
            return entryPoint && filterFile(entryPoint);
          });
          for (const file of outputs) {
            const fileName = path.basename(file);
            const targetPath = path.join(targetDir, fileName);
            if (addBanner) {
              try {
                const inputs = Object.entries(result.metafile.outputs[file].inputs);
                if (!inputs.length) {
                  throw new Error(`No input files found for ${file}`);
                }
                const sourceFile = inputs[0][0];
                if (!sourceFile) {
                  throw new Error(`Invalid source file for ${file}`);
                }
                const content = await promises.readFile(file, "utf-8");
                const manifest = await tryReadManifest(sourceFile, manifestExtensions);
                if (manifest) {
                  const banner = generateScriptableBanner.generateScriptableBanner(manifest);
                  const newContent = banner + "\n" + content;
                  await promises.writeFile(targetPath, newContent);
                  console.log(`Deployed with banner: ${file} -> ${targetPath}`);
                  continue;
                }
              } catch (error) {
                logger.warn(`Failed to add banner to ${file}: ${error}`);
              }
            }
            await promises.copyFile(file, targetPath);
            console.log(`Deployed: ${file} -> ${targetPath}`);
          }
          console.log(`Successfully deployed ${outputs.length} files`);
        } catch (error) {
          console.error(`Deploy failed: ${error.message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    }
  };
}

exports.scriptableDeploy = scriptableDeploy;
