import type {ScriptableManifest} from '@scriptables/manifest';
import {generateScriptableBanner} from '@scriptables/manifest/generateScriptableBanner';
import type {Plugin} from 'esbuild';
import {findICloudDrivePaths, PathType} from 'findicloud';
import {copyFile, readFile, writeFile} from 'fs/promises';
import {basename, join} from 'path';

import {createLogger} from './logger';
import {createPatternMatcher} from './utils';

export interface ScriptableDeployOptions {
  /**
   * Whether to show detailed logs during deployment
   * Default: false
   */
  verbose?: boolean;

  /**
   * Whether to continue building when deployment fails
   * Default: true
   */
  continueOnError?: boolean;

  /**
   * Custom Scriptable directory path
   * If not specified, will automatically search in iCloud Drive
   */
  scriptableDir?: string;

  /**
   * Source file patterns to match entry points
   * Default: ['**\/*.{ts,tsx,js,jsx}']
   * Examples:
   * - '**\/*.widget.ts'         // only deploy widget files
   * - ['**\/*.ts', '!**\/*.test.ts'] // exclude test files
   */
  patterns?: string | string[];

  /**
   * Whether to add Scriptable banner from manifest
   * Default: true
   */
  addBanner?: boolean;

  /**
   * Custom manifest file extensions
   * Default: ['.manifest.json', '.manifest', '.json']
   */
  manifestExtensions?: string[];
}

const findICloudBugsUrl = 'https://github.com/hexxspark/findicloud/issues';

async function tryReadManifest(filePath: string, extensions: string[]): Promise<ScriptableManifest | null> {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, '') + ext;
      const content = await readFile(manifestPath, 'utf-8');
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}

export function scriptableDeploy(options: ScriptableDeployOptions = {}): Plugin {
  const {
    verbose = false,
    continueOnError = true,
    scriptableDir,
    patterns = ['**/*.{ts,tsx,js,jsx}'],
    addBanner = true,
    manifestExtensions = ['.manifest.json', '.manifest', '.json'],
  } = options;

  const filterFile = createPatternMatcher(patterns);

  const logger = createLogger('scriptable-deploy', {
    level: verbose ? 'verbose' : 'info',
  });

  let targetDir: string;

  return {
    name: 'scriptable-deploy',
    setup(build) {
      // Ensure metafile is enabled
      if (!build.initialOptions.metafile) {
        logger.verbose(verbose, 'Automatically enabling metafile option');
        build.initialOptions.metafile = true;
      }

      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            // Find iCloud paths
            const [found] = await findICloudDrivePaths({appNamePattern: 'Scriptable', types: [PathType.APP_STORAGE]});
            targetDir = found?.path;
          }

          if (!targetDir) {
            throw new Error(
              'Scriptable directory not found in iCloud Drive. ' +
                'Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. ' +
                'If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. ' +
                `If the issue persists, please report it at: ${findICloudBugsUrl}`,
            );
          }

          if (verbose) {
            console.log(`[scriptable-deploy] Target directory: ${targetDir}`);
          }
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${(error as Error).message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger.error(message);
        }
      });

      build.onEnd(async result => {
        if (!result.metafile || !targetDir) {
          logger.error('Missing required configuration');
          return;
        }

        try {
          const outputs = Object.keys(result.metafile.outputs).filter(file => {
            const {entryPoint} = result.metafile!.outputs[file];
            return entryPoint && filterFile(entryPoint);
          });

          for (const file of outputs) {
            const fileName = basename(file);
            const targetPath = join(targetDir, fileName);

            if (addBanner) {
              try {
                // Get source file path
                const inputs = Object.entries(result.metafile.outputs[file].inputs);
                if (!inputs.length) {
                  throw new Error(`No input files found for ${file}`);
                }
                const sourceFile = inputs[0][0];
                if (!sourceFile) {
                  throw new Error(`Invalid source file for ${file}`);
                }
                // Read compiled content
                const content = await readFile(file, 'utf-8');

                // Find manifest based on source file path
                const manifest = await tryReadManifest(sourceFile, manifestExtensions);

                if (manifest) {
                  // Generate new content with banner
                  const banner = generateScriptableBanner(manifest);
                  const newContent = banner + '\n' + content;

                  // Write to target file
                  await writeFile(targetPath, newContent);

                  console.log(`Deployed with banner: ${file} -> ${targetPath}`);
                  continue;
                }
              } catch (error) {
                logger.warn(`Failed to add banner to ${file}: ${error}`);
              }
            }

            // If the banner is not needed or the processing fails, copy the file directly
            await copyFile(file, targetPath);

            console.log(`Deployed: ${file} -> ${targetPath}`);
          }

          console.log(`Successfully deployed ${outputs.length} files`);
        } catch (error) {
          console.error(`Deploy failed: ${(error as Error).message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    },
  };
}
